if (typeof(Pandora) != 'object') {
    Pandora = {}
}

if (navigator.geolocation) {
    navigator.geolocation.getCurrentPosition = function(good, bad, opt) {
        if (bad) {
            bad({
                code: 1,
                message: "geolocation not allowed in Pandora",
                PERMISSION_DENIED: 1,
                POSITION_UNAVAILABLE: 2,
                TIMEOUT: 3,
                UNKNOWN_ERROR: 0
                });
        }
    }
    navigator.geolocation.watchPosition = navigator.geolocation.getCurrentPosition;
}

Pandora.urlQueue = [];
Pandora.lastTimestamp = 0;
Pandora.cbCount = 0;
Pandora.playSampleCount = 1;
Pandora.callbacks = new Object();
// TODO: hack until https://jira.savagebeast.com/browse/IOS-5214 gets in
Pandora._multilevelSafeStringify = function (item) {
    var json = JSON.stringify(item);
    // it's unlikely, but item *could* contain the string "[object Object]" before String() is applied;
    // if "[object Object]" is supposed to be there, it will be in the json too
    var isIncorrectlySquashingObjects = (String(item).match(/[object Object]/g) || []).length > (json.match(/[object Object]/g) || []).length
    if (isIncorrectlySquashingObjects) {
        return json;
    } else {
        return String(item);
    }
};
Pandora._callMethod = function (action, qsVals, callback, persistCb) {
    var callbackID = "";
    if (callback) {
        callbackID = (persistCb) ?
        this._addPersistentCallback(callback) : this._addCallback(callback);
    }
    var paramStr = "";
    for (var name in qsVals) {
        var item = qsVals[name]
        if (item != null) {
            paramStr += paramStr ? "&" : "?";
            paramStr += name + "=" + encodeURIComponent(this._multilevelSafeStringify(item));
        }
    }

    var url = "Pandora://" + callbackID + "/" + action + paramStr;
    this.urlQueue.push(url);
    this._checkQueue();
};
Pandora._checkQueue = function () {
    if (Pandora.urlQueue.length > 0) {
        var now = new Date().getTime();
        var elapsed = now - Pandora.lastTimestamp;
        if (elapsed < 500) {
            setTimeout(Pandora._checkQueue, 500 - elapsed);
        } else {
            Pandora.lastTimestamp = now;
            var url = Pandora.urlQueue.shift();
            if (Pandora.getProperty("os") != null) {
                document.location.href = url;
            } else {
                console.log(url);
                // non-mobile browser (for testing)
            }
        }
    }
};
Pandora._addCallback = function (callback) {
    var key = "" + (this.cbCount++);
    this.callbacks[key] = callback;
    return key;
};
Pandora._addPersistentCallback = function (callback) {
    var key = "_" + (this.cbCount++);
    this.callbacks[key] = callback;
    return key;
};
Pandora._methodResponse = function (/*string*/callbackID, /*hash*/response) {
    var callback = this.callbacks[callbackID];
    if (callback && callbackID.indexOf("_") != 0) {
        delete this.callbacks[callbackID];
    }
    if (typeof(callback) == "function" || typeof(callback) == "string") {
        // we need to delay execution a bit to prevent strange (and
        // potentially deadly) interactions with the webview host.
        var _this = this;
        setTimeout(function () {
                   _this._callCallback(callback, [response]);
                   }, 1);
    }
};
Pandora._callCallback = function (nameOrFunction, args) {
    if (typeof nameOrFunction == "function") {
        nameOrFunction.apply(this, args);
    } else {
        // we need to support string names for parity with Android
        window[nameOrFunction].apply(this, args);
    }
};
Pandora.bridgingHasExecuted = true;

if (typeof($) != 'undefined') {
    Pandora.mobileComplete = function () {
        mobileReadyHeld = false;
        $.holdReady(false);
    };
    
    $.holdReady(true);
    var mobileReadyHeld = true;
    setTimeout(Pandora.mobileComplete, 3000);
}

// The following is to redefine some methods/properties that may be needed by the server from the PandoraAPIv2.js
// MOBILE-9559
if (typeof(Pandora.eventHandlers) == "undefined") {
    Pandora.eventHandlers = {};
}

if (typeof(Pandora.registeredForEvents) == "undefined") {
    Pandora.registeredForEvents = false;
}

if (typeof(Pandora.getDiv) == "undefined") {
    Pandora.getDiv = function (divID) {
        if( document.getElementById ) {
            // standard (IE 5+)
            return document.getElementById(divID);
        } else {
            // IE 4
            return document.all[divID];
        }
    };
}

if (typeof(Pandora.registerForEventsIfNecessary) == "undefined") {
    Pandora.registerForEventsIfNecessary = function () {
        if (this.registeredForEvents)
            return;
        
        var div = this.getDiv("tunerEventListenerDiv");
        if( div == null ) {
            div = document.createElement("DIV");
            div.id = "tunerEventListenerDiv";
            if (document.body != null) {
                document.body.appendChild(div);
            }
        }
        this.registeredForEvents = true;
    };
}

if (typeof(Pandora.fireEvent) == "undefined") {
    Pandora.fireEvent = function(eventName, /*Array*/ args) {
        if (this.eventHandlers[eventName] != null) {
            this.eventHandlers[eventName](args);
        }
    };
}

if (typeof(Pandora.setEventHandler) == "undefined") {
    Pandora.setEventHandler = function(event, handler) {
        this.eventHandlers[event] = handler;
        this.registerForEventsIfNecessary();
    };
}

if (typeof(Pandora.getTrackingCodeCookie) == "undefined") {
    Pandora.getTrackingCodeCookie = function() {
        var cookies = document.cookie.split(";");
        for(var i = 0; i < cookies.length; i++) {
            var crumb = cookies[i].split("=");
            if (crumb[0].replace(/^\s\s*/, '') == "tc") {
                return crumb[1];
            }
        }
        return null;
    };
}

if (typeof(Pandora.getBaseUrl) == "undefined") {
    Pandora.getBaseUrl = function() {
        return '<!--#echo var="HOME_PAGE" -->';
    };
}

if (typeof(Pandora.eiWorking) == "undefined") {
    Pandora.eiWorking = true;
}

if (typeof(Pandora.waitForSecretary) == "undefined") {
    Pandora.waitForSecretary = function(callback) {
        callback(true);
    };
}
